Attribute VB_Name = "modRegressioni2"
'=============================================
' Descrizione.....: Routines di regressione
'                   a 2 parametri.
' Nome dei Files..: modRegressioni2.bas
' Data............: 23/9/2000
' Versione........: 1.0 a 32 bits.
' Sistema.........: VB6 sotto Windows NT.
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
'=============================================
'
'   Le regressioni sono calcolate con il metodo
'   dei minimi quadrati e comprendono:
'    REGLIN:    Regressione lineare (1).
'    REGEXP:    Regressione esponenziale (*)(1).
'    REGPOW:    Regressione con elevazione a potenza (*)(1).
'    REGIPE1:   Regressione iperbolica del 1 tipo (*)(1).
'    REGIPE2:   Regressione iperbolica del 2 tipo (*)(1).
'    REGRAT1:   Regressione razionale fratta del 1 tipo (*)(1).
'    REGRAT2:   Regressione razionale fratta del 2 tipo (*)(1).
'    REGSIN:    Regressione sinusoidale (2).
'    (*) con opportune sostituzioni vengono trasformate
'        in una regressione lineare (vedi Trasformazioni).
'
'   e per altri tipi, provare a chiedere...
'
'   (1) da: Esercizi di Analisi Matematica
'           V. Capra
'           Libreria Editrice Universitaria
'           Levrotto & Bella, Torino - 1966.
'   (2) di: F. Languasco
'           Istituto Elettrotecnico Nazionale
'           Galileo Ferraris, Torino - 1974
'
Option Explicit
'
Public Const PI# = 3.14159265358979     ' 4# * Atn(1#)

Public Function RandN(ByVal VMedio!, ByVal Sigma!) As Single
'
'   Ritorna una variabile casuale reale a distribuzione normale.
'   VMedio - 6 * Sigma <= RandN < VMedio + 6 * Sigma:
'
    Dim e!, I&
'
    e = 0!
    For I = 1 To 12
        e = e + Rnd
    Next I
'
    RandN = VMedio + Sigma * (e - 6!)
'
'
'
End Function
Public Function RandU(ByVal V_Min!, ByVal V_Max!) As Single
'
'   Ritorna una variabile casuale reale a distribuzione uniforme.
'   V_Min <= RandU < V_Max:
'
    RandU = V_Min + (V_Max - V_Min) * Rnd
'
'
'
End Function


Public Sub REGLIN(X!(), Y!(), a!, b!, Optional r!)
'
'   Regressione lineare y = a * x + b.
'   Entra con:
'    X():   vettore delle ascisse dei punti da interpolare.
'    Y():   vettore delle ordinate dei punti da interpolare.
'   Ritorna:
'    a, b:  coefficiente angolare ed intercetta
'           della linea interpolante.
'    r:     coefficiente di correlazione.
'
    Dim I&, Np!, N0&, N1&
    Dim Sx!, Sy!, Sxx!, Sxy!, Syy!
'
    N0 = LBound(X)
    N1 = UBound(X)
    Np = CSng(N1 - N0) + 1
'
    For I = N0 To N1
        Sx = Sx + X(I)
        Sy = Sy + Y(I)
        Sxx = Sxx + X(I) * X(I)
        Sxy = Sxy + X(I) * Y(I)
        Syy = Syy + Y(I) * Y(I)
    Next I
'
    a = (Np * Sxy - Sx * Sy) / (Np * Sxx - Sx * Sx)
    b = (Sxx * Sy - Sx * Sxy) / (Np * Sxx - Sx * Sx)
    r = (Np * Sxy - Sx * Sy) / Sqr((Np * Sxx - Sx * Sx) * (Np * Syy - Sy * Sy))
'
'
'
End Sub
Public Sub REGIPE1(X!(), Y!(), a!, b!)
'
'   Regressione iperbolica del 1 tipo y = a / x + b.
'   Entra con:
'    X():   vettore delle ascisse dei punti da interpolare.
'    Y():   vettore delle ordinate dei punti da interpolare.
'   Ritorna:
'    a, b:  parametri della curva interpolante.
'
'   Viene usata la sostituzione:
'    x = 1 / X
'
    Dim I&, Np!, N0&, N1&
    Dim xs!, Sx!, Sy!, Sxx!, Sxy!
'
    N0 = LBound(X)
    N1 = UBound(X)
    Np = CSng(N1 - N0) + 1
'
    For I = N0 To N1
        xs = 1! / X(I)
'
        Sx = Sx + xs
        Sy = Sy + Y(I)
        Sxx = Sxx + xs * xs
        Sxy = Sxy + xs * Y(I)
    Next I
'
    a = (Np * Sxy - Sx * Sy) / (Np * Sxx - Sx * Sx)
    b = (Sxx * Sy - Sx * Sxy) / (Np * Sxx - Sx * Sx)
'
'
'
End Sub
Public Sub REGIPE2(X!(), Y!(), a!, b!)
'
'   Regressione iperbolica del 2 tipo y = b / (x + a).
'   Entra con:
'    X():   vettore delle ascisse dei punti da interpolare.
'    Y():   vettore delle ordinate dei punti da interpolare.
'   Ritorna:
'    a, b:  parametri della curva interpolante.
'
'   Vengono usate le sostituzioni:
'    x = -Y
'    y = X * Y
'
    Dim I&, Np!, N0&, N1&
    Dim xs!, ys!, Sx!, Sy!, Sxx!, Sxy!
'
    N0 = LBound(X)
    N1 = UBound(X)
    Np = CSng(N1 - N0) + 1
'
    For I = N0 To N1
        xs = -Y(I)
        ys = X(I) * Y(I)
'
        Sx = Sx + xs
        Sy = Sy + ys
        Sxx = Sxx + xs * xs
        Sxy = Sxy + xs * ys
    Next I
'
    a = (Np * Sxy - Sx * Sy) / (Np * Sxx - Sx * Sx)
    b = (Sxx * Sy - Sx * Sxy) / (Np * Sxx - Sx * Sx)
'
'
'
End Sub
Public Sub REGRAT1(X!(), Y!(), a!, b!)
'
'   Regressione razionale fratta del 1 tipo y = b * x / (x + a).
'   Entra con:
'    X():   vettore delle ascisse dei punti da interpolare.
'    Y():   vettore delle ordinate dei punti da interpolare.
'   Ritorna:
'    a, b:  parametri della curva interpolante.
'
'   Viene usata la sostituzione:
'    x = -Y / X
'
    Dim I&, Np!, N0&, N1&
    Dim xs!, Sx!, Sy!, Sxx!, Sxy!
'
    N0 = LBound(X)
    N1 = UBound(X)
    Np = CSng(N1 - N0) + 1
'
    For I = N0 To N1
        xs = -Y(I) / X(I)
'
        Sx = Sx + xs
        Sy = Sy + Y(I)
        Sxx = Sxx + xs * xs
        Sxy = Sxy + xs * Y(I)
    Next I
'
    a = (Np * Sxy - Sx * Sy) / (Np * Sxx - Sx * Sx)
    b = (Sxx * Sy - Sx * Sxy) / (Np * Sxx - Sx * Sx)
'
'
'
End Sub
Public Sub REGPOW(X!(), Y!(), k!, a!)
'
'   Regressione con elevazione a potenza y = k * (x ^ a)
'   Entra con:
'    X():   vettore delle ascisse dei punti da interpolare.
'    Y():   vettore delle ordinate dei punti da interpolare.
'           per tutti i punti deve essere X(I) > 0 e Y(I) > 0.
'   Ritorna:
'    k, a:  parametri della curva interpolante.
'
'   Vengono usate le sostituzioni:
'    x = Log(X)
'    y = Log(Y)
'    b = Log(k)
'
    Dim I&, Np!, N0&, N1&
    Dim xs!, ys!, Sx!, Sy!, Sxx!, Sxy!
'
    N0 = LBound(X)
    N1 = UBound(X)
    Np = CSng(N1 - N0) + 1
'
    For I = N0 To N1
        xs = Log(X(I))
        ys = Log(Y(I))
'
        Sx = Sx + xs
        Sy = Sy + ys
        Sxx = Sxx + xs * xs
        Sxy = Sxy + xs * ys
    Next I
'
    a = (Np * Sxy - Sx * Sy) / (Np * Sxx - Sx * Sx)
    k = Exp((Sxx * Sy - Sx * Sxy) / (Np * Sxx - Sx * Sx))
'
'
'
End Sub
Private Sub Trasformazioni()
'
'   Esempio di trasformazione della regressione
'   esponenziale in una lineare:
'
'   Data la Y = k * Exp(a * X) essa si puo' scrivere
'   come Log(Y) = a * X + Log(k).
'   Per quest' ultima si possono usare le formule della
'   regressione lineare y = a * x + b con le sostituzioni:
'    x = X
'    y = Log(Y)
'    b = Log(k)
'
'
'
End Sub
Public Sub REGRAT2(X!(), Y!(), a!, b!)
'
'   Regressione razionale fratta del 2 tipo y = b / (a + x ^ 2).
'   Entra con:
'    X():   vettore delle ascisse dei punti da interpolare.
'    Y():   vettore delle ordinate dei punti da interpolare.
'   Ritorna:
'    a, b:  parametri della curva interpolante.
'
'   Vengono usate le sostituzioni:
'    x = -Y
'    y = Y * (X ^ 2)
'
    Dim I&, Np!, N0&, N1&
    Dim xs!, ys!, Sx!, Sy!, Sxx!, Sxy!
'
    N0 = LBound(X)
    N1 = UBound(X)
    Np = CSng(N1 - N0) + 1
'
    For I = N0 To N1
        xs = -Y(I)
        ys = Y(I) * (X(I) ^ 2)
'
        Sx = Sx + xs
        Sy = Sy + ys
        Sxx = Sxx + xs * xs
        Sxy = Sxy + xs * ys
    Next I
'
    a = (Np * Sxy - Sx * Sy) / (Np * Sxx - Sx * Sx)
    b = (Sxx * Sy - Sx * Sxy) / (Np * Sxx - Sx * Sx)
'
'
'
End Sub
Public Sub REGEXP(X!(), Y!(), k!, a!)
'
'   Regressione esponenziale y = k * Exp(a * x).
'   Entra con:
'    X():   vettore delle ascisse dei punti da interpolare.
'    Y():   vettore delle ordinate dei punti da interpolare;
'           per tutti i punti deve essere Y(I) > 0.
'   Ritorna:
'    k, a:  parametri della curva interpolante.
'
'   Vengono usate le sostituzioni:
'    y = Log(Y)
'    b = Log(K)
'
    Dim I&, Np!, N0&, N1&
    Dim ys!, Sx!, Sy!, Sxx!, Sxy!
'
    N0 = LBound(X)
    N1 = UBound(X)
    Np = CSng(N1 - N0) + 1
'
    For I = N0 To N1
        ys = Log(Y(I))
'
        Sx = Sx + X(I)
        Sy = Sy + ys
        Sxx = Sxx + X(I) * X(I)
        Sxy = Sxy + X(I) * ys
    Next I
'
    a = (Np * Sxy - Sx * Sy) / (Np * Sxx - Sx * Sx)
    k = Exp((Sxx * Sy - Sx * Sxy) / (Np * Sxx - Sx * Sx))
'
'
'
End Sub
Public Sub REGSIN(X!(), Y!(), ByVal omega!, a!, fase!)
'
'   Regressione sinusoidale y = a * Sin(omega * x + fase).
'   Entra con:
'    X():       vettore delle ascisse dei punti da interpolare.
'    Y():       vettore delle ordinate dei punti da interpolare.
'    omega:     pulsazione della sinusoide interpolante.
'   Ritorna:
'    a, fase:   ampiezza e fase della sinusoide interpolante.
'
    Dim I&, N0&, N1&
    Dim Op!, Sf!, Cf!
    Dim Sysx!, Ssxcx!, Sycx!, Ssxsx!, Scxcx!
'
    N0 = LBound(X)
    N1 = UBound(X)
'
    For I = N0 To N1
        Op = omega * X(I)
        Sysx = Sysx + Y(I) * Sin(Op)
        Ssxcx = Ssxcx + Sin(Op) * Cos(Op)
        Sycx = Sycx + Y(I) * Cos(Op)
        Ssxsx = Ssxsx + Sin(Op) * Sin(Op)
        Scxcx = Scxcx + Cos(Op) * Cos(Op)
    Next I
'
    fase = Atn((Sysx * Ssxcx - Sycx * Ssxsx) / (Sycx * Ssxcx - Sysx * Scxcx))
    Sf = Sin(fase)
    Cf = Cos(fase)
    a = (Cf * Sysx + Sf * Sycx) _
      / (Cf * Cf * Ssxsx + Sf * Sf * Scxcx + 2! * Sf * Cf * Ssxcx)
'
    If a < 0! Then
        a = -a
        fase = (fase + PI)
    End If
    If fase < 0! Then fase = fase + 2! * PI
'
'
'
End Sub

